/*
 */
package booknaviger.readinterface;

import booknaviger.searcher.TableSearcher;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;

/**
 * This dialog show the pages of the currently read album
 * @author Inervo
 */
public class ListPagesDialog extends javax.swing.JDialog {
    ReadInterface readInterface = null;

    /**
     * Creates new form ListPagesDialog
     * @param readInterface 
     */
    public ListPagesDialog(ReadInterface readInterface) {
        super(readInterface, true);
        Logger.getLogger(ListPagesDialog.class.getName()).entering(ListPagesDialog.class.getName(), "ListPagesDialog");
        this.readInterface = readInterface;
        initComponents();
        setLocationRelativeTo(null);
        Logger.getLogger(ListPagesDialog.class.getName()).exiting(ListPagesDialog.class.getName(), "ListPagesDialog");
    }
    
    /**
     * Fill the table with the pages name and select the current page
     * @param pagesName The pages name
     * @param currentPage The index of the current page
     */
    protected void fillPagesName(final List<String> pagesName, final int currentPage) {
        Logger.getLogger(ListPagesDialog.class.getName()).entering(ListPagesDialog.class.getName(), "fillPagesName", new Object[] {pagesName, currentPage});
        final DefaultTableModel dtm = (DefaultTableModel) pagesListTable.getModel();
        List<Thread> rows = new ArrayList<>();
        for (int i = 0; i < pagesName.size(); i++) {
            final int index = i;
            Thread tampon = new Thread(new Runnable() {

                @Override
                public void run() {
                    dtm.addRow(new String[] {pagesName.get(index)});
                }
            });
            SwingUtilities.invokeLater(tampon);
            rows.add(tampon);
        }
        for (Thread thread : rows) {
            try {
                thread.join();
            } catch (InterruptedException ex) {
                Logger.getLogger(ListPagesDialog.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
        SwingUtilities.invokeLater(new Runnable() {

            @Override
            public void run() {
                pagesListTable.getSelectionModel().setSelectionInterval(currentPage, currentPage);
                Rectangle cellRect = pagesListTable.getCellRect(currentPage, 0, true);
                cellRect.y += 1;
                pagesListTable.scrollRectToVisible(cellRect);
            }
        });
        Logger.getLogger(ListPagesDialog.class.getName()).exiting(ListPagesDialog.class.getName(), "fillPagesName");
    }

    /**
     * This method is called from within the constructor to initialize the
     * form. WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        pagesListScrollPane = new javax.swing.JScrollPane();
        pagesListTable = new javax.swing.JTable();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        pagesListTable.setBackground(new java.awt.Color(0, 0, 0));
        pagesListTable.setForeground(new java.awt.Color(255, 255, 255));
        pagesListTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Pages"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        pagesListTable.setFillsViewportHeight(true);
        pagesListTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        pagesListTable.setShowHorizontalLines(false);
        pagesListTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                pagesListTableMouseClicked(evt);
            }
        });
        pagesListTable.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                pagesListTableKeyPressed(evt);
            }
        });
        pagesListScrollPane.setViewportView(pagesListTable);
        new TableSearcher(pagesListTable, this).activateQuickSearch();

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(pagesListScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
            .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                .add(pagesListScrollPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 300, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Event if a key is pressed while on the pages list
     * @param evt the event associated (and key pressed)
     */
    private void pagesListTableKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_pagesListTableKeyPressed
        Logger.getLogger(ListPagesDialog.class.getName()).entering(ListPagesDialog.class.getName(), "pagesListTableKeyPressed");
        if (evt.getKeyCode() == KeyEvent.VK_ESCAPE) {
            this.dispose();
        }
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            evt.consume();
            goReadSelectedPage();
        }
        Logger.getLogger(ListPagesDialog.class.getName()).exiting(ListPagesDialog.class.getName(), "pagesListTableKeyPressed");
    }//GEN-LAST:event_pagesListTableKeyPressed

    /**
     * Mouse click event while on the pagesList
     * @param evt The associated event
     */
    private void pagesListTableMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_pagesListTableMouseClicked
        Logger.getLogger(ListPagesDialog.class.getName()).entering(ListPagesDialog.class.getName(), "pagesListTableMouseClicked");
        if (evt.getClickCount() == 2) {
            goReadSelectedPage();
        }
        Logger.getLogger(ListPagesDialog.class.getName()).exiting(ListPagesDialog.class.getName(), "pagesListTableMouseClicked");
    }//GEN-LAST:event_pagesListTableMouseClicked

    /**
     * Go to the selected page on the table
     */
    private void goReadSelectedPage() {
        Logger.getLogger(ListPagesDialog.class.getName()).entering(ListPagesDialog.class.getName(), "goReadSelectedPage");
        new Thread(new Runnable() {

            @Override
            public void run() {
                readInterface.goPage(pagesListTable.getSelectionModel().getMinSelectionIndex() + 1);
                Logger.getLogger(ListPagesDialog.class.getName()).exiting(ListPagesDialog.class.getName(), "goReadSelectedPage");
            }
        }).start();
        this.dispose();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JScrollPane pagesListScrollPane;
    private javax.swing.JTable pagesListTable;
    // End of variables declaration//GEN-END:variables
}
